


PROGRAM "fire"
VERSION "0.1"
'CONSOLE

	IMPORT "msvcrt"
	IMPORT "gdi32"
	IMPORT "user32"
	IMPORT "kernel32"

	IMPORT "SDL"
'	IMPORT "SDL_image"	' SDL must be imported before SDL_image
'	IMPORT "SDL_ttf"


TYPE ColorHSL
	UBYTE	.h
	UBYTE	.s
	UBYTE	.l
END TYPE

TYPE ColorRGB
	UBYTE	.r
	UBYTE	.g
	UBYTE	.b
END TYPE

DECLARE FUNCTION main ()

' set 32bit pixel colour
DECLARE FUNCTION VOID SetPixelRGB (SDL_Surface surface, int x, int y, int r, int g, int b)
' set 32bit pixel colour
DECLARE FUNCTION VOID SetPixelColour (SDL_Surface surface, int x, int y, lpSDL_Colour colour)

DECLARE FUNCTION SDL_BlitSurface (lpSDL_Surface src, lpSDL_Rect srcrect, lpSDL_Surface dst, lpSDL_Rect dstrect)

DECLARE FUNCTION lpSDL_Colour GetPixelColour (SDL_Surface surface, int x, int y)
DECLARE FUNCTION lpSDL_Surface SDL_LoadBMP (lpfilename)

DECLARE FUNCTION XLONG HSLtoRGB(float h, float s, float l)

DECLARE FUNCTION pollevent()

FUNCTION main ()

    'set up the screen
	lpSDL_Surface lpscreen, lpimage
	lpSDL_PixelFormat lppixformat
	SDL_Surface screen
	SDL_PixelFormat pixformat
	SDL_Rect rect
    float time, oldTime 'the time of this and the previous frame, for timing

	XLONG fire[]  'this buffer will contain the fire
	XLONG palette[] 'this will contain the color palette

   	screenWidth = 512
	screenHeight = 128
 	w = screenWidth
 	h = screenHeight

	DIM fire[w+1,h+1]  'this buffer will contain the fire
	DIM palette[256] 'this will contain the color palette


	IF (SDL_Init($$SDL_INIT_VIDEO))
		RETURN 1
	END IF

	lpscreen = SDL_SetVideoMode (screenWidth, screenHeight, 32, $$SDL_HWSURFACE | $$SDL_DOUBLEBUF)
	IF (!lpscreen)
		SDL_Quit()
		RETURN 1
	ELSE
		memcpy(&screen,lpscreen,SIZE(screen))
	END IF

	SDL_WM_SetCaption(&"SDL Fire", 0)

    'generate the palette
    FOR x = 0 TO 255
        'HSLtoRGB is used to generate colors:
        'Hue goes from 0 to 85: red to yellow
        'Saturation is always the maximum: 255
        'Lightness is 0..255 for x=0..128, and 255 for x=128..255

        'set the palette to the calculated RGB value
		palette[x] = HSLtoRGB(x / 3, 255, MIN(255, x * 2))
    NEXT x

    DO WHILE(!pollevent())

        'randomize the bottom row of the fire buffer
        FOR x = 0 TO w-1
        	fire[x,h - 1] = abs(32768 + rand()) & 255
        NEXT x

        'do the fire calculations for every pixel, from top to bottom
    	FOR y = 0 TO h-1
			FOR x = 0 TO w-1
				'fire[x,y] = ((fire[(x - 1 + w) MOD w,(y + 1) MOD h] + fire[(x) MOD w,(y + 1) MOD h] + fire[(x + 1) MOD w,(y + 1) MOD h] + fire[(x) MOD w,(y + 2) MOD h]) * 32) / 129

				'as 'h' (screenHeight) == 128 which is a power of 2 the '&' operator can be used in place of MOD
				' ie, 'n MOD w' becomes 'n & 127': 'n MOD w' becomes 'n & 511'
				fire[x,y] = ((fire[(x - 1 + w) & 511,(y + 1) & 127] + fire[(x) & 511,(y + 1) & 127] + fire[(x + 1) & 511,(y + 1) & 127] + fire[(x) & 511,(y + 2) & 127]) * 32) / 129
				SetPixelColour(screen, x, y, palette[fire[x,y]])
        	NEXT x
        NEXT y

    	SDL_Flip(lpscreen)
        Sleep(40)
    LOOP

	SDL_Quit()
	RETURN 0
END FUNCTION


'Converts an HSL color to RGB color
 'this function works with floats between 0 and 1
FUNCTION XLONG HSLtoRGB(float h, float s, float l)

    float r, g, b
    float temp1, temp2, tempr, tempg, tempb
    UBYTE rr,gg,bb

    h = h / 256.0
    s = s / 256.0
    l = l / 256.0

    'If saturation is 0, the color is a shade of grey
    IF (s == 0)
    	r = l
    	g = l
    	b = l
    ELSE	' If saturation > 0, more complex calculations are needed
        'set the temporary values
        IF (l < 0.5)
        	temp2 = l * (1 + s)
        ELSE
        	temp2 = (l + s) - (l * s)
        END IF

        temp1 = 2 * l - temp2
        tempr=h + 1.0 / 3.0

        IF (tempr > 1.0) tempr = tempr - 1

        tempg = h
        tempb = h - 1.0 / 3.0
        IF (tempb < 0.0) tempb = tempb + 1

        'red
        IF (tempr < 1.0 / 6.0)
        	r = temp1 + (temp2 - temp1) * 6.0 * tempr
        ELSE
        	IF (tempr < 0.5)
        		r = temp2
        	ELSE
				IF (tempr < 2.0 / 3.0)
        			r = temp1 + (temp2 - temp1) * ((2.0 / 3.0) - tempr) * 6.0
        		ELSE
        			r = temp1
        		END IF
        	END IF
		END IF

         'green
        IF (tempg < 1.0 / 6.0)
        	g = temp1 + (temp2 - temp1) * 6.0 * tempg
        ELSE
        	IF (tempg < 0.5)
        		g=temp2
        	ELSE
        		IF (tempg < 2.0 / 3.0)
        			g = temp1 + (temp2 - temp1) * ((2.0 / 3.0) - tempg) * 6.0
        		ELSE
        			g = temp1
        		END IF
        	END IF
        END IF

        'blue
        IF (tempb < 1.0 / 6.0)
        	b = temp1 + (temp2 - temp1) * 6.0 * tempb
        ELSE
        	IF (tempb < 0.5)
        		b = temp2
        	ELSE
        		IF (tempb < 2.0 / 3.0)
        			b = temp1 + (temp2 - temp1) * ((2.0 / 3.0) - tempb) * 6.0
        		ELSE
        			b = temp1
        		END IF
        	END IF
        END IF
    END IF

	rr = XLONG(r * 255.0)
	gg = XLONG(g * 255.0)
	bb = XLONG(b * 255.0)
    RETURN (rr << 16)|(gg << 8)|bb

END FUNCTION


FUNCTION lpSDL_Surface SDL_LoadBMP (lpfilename)

	RETURN SDL_LoadBMP_RW(SDL_RWFromFile(lpfilename, &"rb"), 1)
END FUNCTION

FUNCTION SDL_BlitSurface (lpSDL_Surface src, lpSDL_Rect srcrect, lpSDL_Surface dst, lpSDL_Rect dstrect)

	RETURN SDL_UpperBlit(src, srcrect, dst, dstrect)
END FUNCTION


' 32bit surface
FUNCTION VOID SetPixelRGB (SDL_Surface surface, int x, int y, int r, int g, int b)

	'XLONGAT(surface.pixels + (y*(surface.w<<2)) + (x<<2)) = SDL_MapRGB(surface.format, r, g, b)
	' or to save a function call...
	XLONGAT(surface.pixels + (y*surface.pitch) + (x << 2)) = (r << 16)|(g << 8)|b
END FUNCTION

' 32bit surface
FUNCTION VOID SetPixelColour (SDL_Surface surface, int x, int y, lpSDL_Colour colour)

	XLONGAT(surface.pixels + (y*surface.pitch) + (x<<2)) = colour
END FUNCTION


' 24bit surface
FUNCTION lpSDL_Colour GetPixelColour (SDL_Surface surface, int x, int y)

	RETURN XLONGAT(surface.pixels + y * surface.pitch + x * 3)
END FUNCTION


FUNCTION pollevent()

    SDL_Event event

    IF (!SDL_PollEvent(&event)) RETURN 0
    IF (event.type == $$SDL_QUIT) RETURN 1

    lpinkeys = SDL_GetKeyState(0)
	IF (lpinkeys)
		IF (UBYTEAT(lpinkeys + $$SDLK_ESCAPE)) RETURN 1
	END IF

    RETURN 0
END FUNCTION


END PROGRAM