

' Program to load a wave file and loop playing it using SDL sound 
' This is only for simple WAVEs


PROGRAM "loopwave"
VERSION "0.1"
CONSOLE

	IMPORT "msvcrt"
	IMPORT "gdi32"
	IMPORT "user32"
	IMPORT "kernel32"

	IMPORT "SDL"
	'IMPORT "SDL_image"	' SDL must be imported before SDL_image
	'IMPORT "SDL_ttf"
	
	
	
$$SIGINT	=	2	' Interactive attention 
$$SIGILL	=	4	' Illegal instruction 
$$SIGFPE	=	8	' Floating point error 
$$SIGSEGV	=	11	' Segmentation violation 
$$SIGTERM	=	15	' Termination request 
$$SIGBREAK	=	21	' Control-break 
$$SIGABRT	=	22	' Abnormal termination (abort) 
	
DECLARE FUNCTION main ()
DECLARE CFUNCTION VOID poked (XLONG sig)
DECLARE CFUNCTION fillerup (unused, lpstream, len)

DECLARE CFUNCTION SDL_LoadWAV(file, spec, audio_buf, audio_len)



TYPE WAVE
	SDL_AudioSpec	.spec
	XLONG			.sound			' Pointer to wave data 
	Uint32			.soundlen		' Length of wave data 
	int				.soundpos		' Current play position 
END TYPE



FUNCTION main()

	STRING name
	SHARED WAVE wave
	SHARED done
	
	done = 0
	name = NULL$(128)

	' Load the SDL library 
	IF (SDL_Init($$SDL_INIT_AUDIO) < 0 )
		PRINT "Couldn't initialize SDL:", CSTRING$(SDL_GetError())
		RETURN
	END IF

	' Load the wave file into memory 
	IF (SDL_LoadWAV(&"sample.wav",&wave.spec, &wave.sound, &wave.soundlen) == 0 )
		PRINT "Couldn't load sample:", CSTRING$(SDL_GetError())
		RETURN
	END IF
	
	wave.spec.lpcallback = &fillerup()

	' Set the signals 
	signal($$SIGINT, &poked())
	signal($$SIGBREAK, &poked())
	signal($$SIGTERM, &poked())

	' Initialize fillerup() variables 
	IF (SDL_OpenAudio(&wave.spec, NULL) < 0 )
		PRINT "Couldn't open audio:", CSTRING$(SDL_GetError())
		SDL_FreeWAV(wave.sound)
		RETURN
	END IF
	
	SDL_PauseAudio(0)

	' Let the audio run 
	PRINT "Using audio driver:", CSTRING$(SDL_AudioDriverName(&name, 32))
	
	DO WHILE (!done && (SDL_GetAudioStatus() == $$SDL_AUDIO_PLAYING) )
		SDL_Delay(500)
	LOOP

	PRINT "exiting.."
	' Clean up on signal 
	
	SDL_PauseAudio(0)
	SDL_CloseAudio()
	SDL_FreeWAV(wave.sound)
	SDL_Quit()
	RETURN
END FUNCTION

CFUNCTION SDL_LoadWAV(lpfile, spec, audio_buf, audio_len)

	RETURN SDL_LoadWAV_RW(SDL_RWFromFile(lpfile, &"rb"),1, spec, audio_buf, audio_len)
END FUNCTION


CFUNCTION fillerup (unused, stream, len)
	SHARED WAVE wave
	
	XLONG waveptr
	int waveleft

	' Set up the pointers 
	waveptr = wave.sound + wave.soundpos
	waveleft = wave.soundlen - wave.soundpos

	' Go! 
	DO WHILE ( waveleft <= len )
		SDL_MixAudio(stream, waveptr, waveleft, $$SDL_MIX_MAXVOLUME)
		stream = stream + waveleft
		len = len - waveleft
		waveptr = wave.sound
		waveleft = wave.soundlen
		wave.soundpos = 0
	LOOP
	
	SDL_MixAudio(stream, waveptr, len, $$SDL_MIX_MAXVOLUME)
	wave.soundpos = wave.soundpos + len
	
END FUNCTION


CFUNCTION VOID poked (XLONG sig)
	SHARED done

	done = 1
END FUNCTION

END PROGRAM

